package org.processmining.directlyfollowsmodelminer.mining.plugins;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.directlyfollowsmodelminer.model.DirectlyFollowsModel;
import org.processmining.directlyfollowsmodelminer.model.DirectlyFollowsModelImplQuadratic;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;

@Plugin(name = "Import a directly follows model", parameterLabels = { "Filename" }, returnLabels = {
		"Directly follows model" }, returnTypes = { DirectlyFollowsModel.class })
@UIImportPlugin(description = "Directly follows model files", extensions = { "dfm" })
public class DfmImportPlugin extends AbstractImportPlugin {

	private static final int BUFFER_SIZE = 8192 * 4;
	private static final String CHARSET = Charset.defaultCharset().name();

	public DirectlyFollowsModel importFromStream(PluginContext context, InputStream input, String filename,
			long fileSizeInBytes) throws Exception {

		//read the file
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024];
		int len;
		while ((len = input.read(buffer)) > -1) {
			baos.write(buffer, 0, len);
		}
		baos.flush();

		//try 2
		DirectlyFollowsModel dfg2 = readFile(new ByteArrayInputStream(baos.toByteArray()));

		if (dfg2 != null) {
			return dfg2;
		}

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(null, "Invalid directly follows model/minimum self-distance graph file",
						"Invalid file", JOptionPane.ERROR_MESSAGE);
			}
		});
		context.getFutureResult(0).cancel(false);
		return null;
	}

	public static DirectlyFollowsModel readFile(InputStream input) throws IOException {
		BufferedReader r = new BufferedReader(new InputStreamReader(input, CHARSET), BUFFER_SIZE);

		//read header
		String header = r.readLine();
		if (!header.equals(DfmExportPlugin.header)) {
			return null;
		}

		//read activity names
		int nrOfActivities = Integer.parseInt(r.readLine());
		String[] activities = new String[nrOfActivities];
		for (int i = 0; i < nrOfActivities; i++) {
			activities[i] = r.readLine();
		}

		DirectlyFollowsModel dfm = new DirectlyFollowsModelImplQuadratic();
		dfm.addActivities(activities);

		//read start activities
		{
			int nrOfStartActivities = Integer.parseInt(r.readLine());
			for (int i = 0; i < nrOfStartActivities; i++) {
				int activityIndex = Integer.parseInt(r.readLine());
				dfm.addStartActivity(activityIndex);
			}
		}

		//read end activities
		{
			int nrOfEndActivities = Integer.parseInt(r.readLine());
			for (int i = 0; i < nrOfEndActivities; i++) {
				int activityIndex = Integer.parseInt(r.readLine());
				dfm.addEndActivity(activityIndex);
			}
		}

		//read dfg-edges
		{
			int nrOfEdges = Integer.parseInt(r.readLine());
			for (int i = 0; i < nrOfEdges; i++) {
				String line = r.readLine();
				int eAt = line.indexOf('>');
				int source = Integer.parseInt(line.substring(0, eAt));
				int target = Integer.parseInt(line.substring(eAt + 1));

				dfm.addEdge(source, target);
			}
		}

		return dfm;
	}
}
